@@ -24,6 +24,7 @@ use crate::{
         },
         change_ifaces,
         ethtool::EthtoolInfo,
+        ipoib::{get_ipoib_info, IpoibInfo},
         mac_vlan::{get_mac_vlan_info, MacVlanInfo},
         mac_vtap::{get_mac_vtap_info, MacVtapInfo},
         sriov::{get_sriov_info, SriovInfo},
@@ -52,6 +53,7 @@ use std::collections::HashMap;
 
 #[derive(Serialize, Deserialize, Debug, PartialEq, Clone)]
 #[serde(rename_all = "snake_case")]
+#[non_exhaustive]
 pub enum IfaceType {
     Bond,
     Veth,
@@ -66,6 +68,7 @@ pub enum IfaceType {
     MacVlan,
     MacVtap,
     OpenvSwitch,
+    Ipoib,
     Unknown,
     Other(String),
 }
@@ -76,8 +79,36 @@ impl Default for IfaceType {
     }
 }
 
+impl std::fmt::Display for IfaceType {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        write!(
+            f,
+            "{}",
+            match self {
+                Self::Bond => "bond",
+                Self::Veth => "veth",
+                Self::Bridge => "bridge",
+                Self::Vlan => "vlan",
+                Self::Dummy => "dummy",
+                Self::Vxlan => "vxlan",
+                Self::Loopback => "loopback",
+                Self::Ethernet => "ethernet",
+                Self::Vrf => "vrf",
+                Self::Tun => "tun",
+                Self::MacVlan => "macvlan",
+                Self::MacVtap => "macvtap",
+                Self::OpenvSwitch => "openvswitch",
+                Self::Ipoib => "ipoib",
+                Self::Unknown => "unknown",
+                Self::Other(s) => s,
+            }
+        )
+    }
+}
+
 #[derive(Serialize, Deserialize, Debug, PartialEq, Clone)]
 #[serde(rename_all = "snake_case")]
+#[non_exhaustive]
 pub enum IfaceState {
     Up,
     Dormant,
@@ -114,6 +145,7 @@ impl std::fmt::Display for IfaceState {
 
 #[derive(Serialize, Deserialize, Debug, PartialEq, Clone)]
 #[serde(rename_all = "snake_case")]
+#[non_exhaustive]
 pub enum IfaceFlags {
     AllMulti,
     AutoMedia,
@@ -143,6 +175,7 @@ impl Default for IfaceFlags {
 
 #[derive(Serialize, Deserialize, Debug, PartialEq, Clone)]
 #[serde(rename_all = "snake_case")]
+#[non_exhaustive]
 pub enum ControllerType {
     Bond,
     Bridge,
@@ -165,6 +198,7 @@ impl From<&str> for ControllerType {
 }
 
 #[derive(Serialize, Deserialize, Debug, PartialEq, Clone, Default)]
+#[non_exhaustive]
 pub struct Iface {
     pub name: String,
     #[serde(skip_serializing)]
@@ -172,6 +206,10 @@ pub struct Iface {
     pub iface_type: IfaceType,
     pub state: IfaceState,
     pub mtu: i64,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub min_mtu: Option<i64>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub max_mtu: Option<i64>,
     pub flags: Vec<IfaceFlags>,
     #[serde(skip_serializing_if = "Option::is_none")]
     pub ipv4: Option<Ipv4Info>,
@@ -213,6 +251,8 @@ pub struct Iface {
     pub mac_vtap: Option<MacVtapInfo>,
     #[serde(skip_serializing_if = "Option::is_none")]
     pub sriov: Option<SriovInfo>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub ipoib: Option<IpoibInfo>,
 }
 
 // TODO: impl From Iface to IfaceConf
@@ -251,6 +291,12 @@ pub(crate) fn parse_nl_msg_to_iface(
     for nla in &nl_msg.nlas {
         if let Nla::Mtu(mtu) = nla {
             iface_state.mtu = *mtu as i64;
+        } else if let Nla::MinMtu(mtu) = nla {
+            iface_state.min_mtu =
+                if *mtu != 0 { Some(*mtu as i64) } else { None };
+        } else if let Nla::MaxMtu(mtu) = nla {
+            iface_state.max_mtu =
+                if *mtu != 0 { Some(*mtu as i64) } else { None };
         } else if let Nla::Address(mac) = nla {
             mac_len = Some(mac.len());
             iface_state.mac_address = parse_as_mac(mac.len(), mac)?;
@@ -277,6 +323,7 @@ pub(crate) fn parse_nl_msg_to_iface(
                         nlas::InfoKind::Vrf => IfaceType::Vrf,
                         nlas::InfoKind::MacVlan => IfaceType::MacVlan,
                         nlas::InfoKind::MacVtap => IfaceType::MacVtap,
+                        nlas::InfoKind::Ipoib => IfaceType::Ipoib,
                         nlas::InfoKind::Other(s) => match s.as_ref() {
                             "openvswitch" => IfaceType::OpenvSwitch,
                             _ => IfaceType::Other(s.clone()),
@@ -311,6 +358,9 @@ pub(crate) fn parse_nl_msg_to_iface(
                         IfaceType::MacVtap => {
                             iface_state.mac_vtap = get_mac_vtap_info(d)?
                         }
+                        IfaceType::Ipoib => {
+                            iface_state.ipoib = get_ipoib_info(d);
+                        }
                         _ => log::warn!(
                             "Unhandled IFLA_INFO_DATA for iface type {:?}",
                             iface_state.iface_type
@@ -353,7 +403,7 @@ pub(crate) fn parse_nl_msg_to_iface(
                 }
             }
         } else if let Nla::VfInfoList(data) = nla {
-            if let Ok(info) = get_sriov_info(data, mac_len) {
+            if let Ok(info) = get_sriov_info(&iface_state.name, data, mac_len) {
                 iface_state.sriov = Some(info);
             }
         } else {
@@ -406,11 +456,10 @@ pub(crate) fn fill_bridge_vlan_info(
     if name.is_empty() {
         return Ok(());
     }
-    if let Some(mut iface_state) = iface_states.get_mut(&name) {
+    if let Some(iface_state) = iface_states.get_mut(&name) {
         for nla in &nl_msg.nlas {
             if let Nla::AfSpecBridge(data) = nla {
-                parse_bridge_vlan_info(&mut iface_state, data)?;
-                break;
+                parse_bridge_vlan_info(iface_state, data)?;
             }
         }
     }
@@ -483,6 +532,7 @@ fn _parse_iface_flags(flags: u32) -> Vec<IfaceFlags> {
 }
 
 #[derive(Serialize, Deserialize, Debug, PartialEq, Clone, Default)]
+#[non_exhaustive]
 pub struct IfaceConf {
     pub name: String,
     #[serde(default = "default_iface_state_in_conf")]
